//
// Simulates an Afb measurement with distribution (1 + a*x + x**2) * 3 / 4 and
// constant efficiency for abs(x) < x_max and decreasing lineraly to 0 for
// abs(x) > x_max

#include "TF1.h"
#include "TH1F.h"
#include "TMath.h"
#include "TRandom.h"


// The FB distribution function
Double_t fbdist(Double_t *x, Double_t *par)
{
   // par[0] is the FB asymmetry parameter

   Double_t f = 0.;
   if (TMath::Abs(x[0]) < 1.)
      f = (1. + par[0]*x[0] + x[0]*x[0]) * 0.75;
   return f;
}

// The efficiency function
Double_t eff(Double_t *x, Double_t *par)
{
   // par[0] is the plateau efficiency, par[1] is the max x defining the plateau

   Double_t e = par[0];
   if (TMath::Abs(x[0]) > par[1]) {
      Double_t dx = 1. - par[1];
      e = par[0] - (TMath::Abs(x[0]) - par[1]) * par[0] / dx;
   }
   return e;
}

TH1F *hfb = 0;
TF1 *fbfun = 0, *effun = 0;

void afb() {

   // Create function for the FB distribution with parameter set to 0.2
   fbfun = new TF1("fbfun",fbdist,-1.,1.,1);
   fbfun->SetParameter(0,.2);
   Printf("AFB param : %f", fbfun->GetParameter(0));

   // Create function for the efficiency with parameters 0.95 (plateau) and 0.9 (range)
   effun = new TF1("effun",eff,-1.,1.,2);
   effun->SetParameter(0,.95);
   effun->SetParameter(1,.85);
   Printf("EFF param : %f %f", effun->GetParameter(0), effun->GetParameter(1));

   // Create histo with 20 bins in [-1.,1.]
   hfb = new TH1F("hfb", "hfb", 20, -1., 1.);

   // Fill histo with 100000 events applying the efficiency
   Int_t nevt = 100000;
   FILE *fout = fopen("evts.txt","w");
   while (nevt--) {
      Double_t xx = fbfun->GetRandom();
      Double_t ww = gRandom->Rndm();
      if (effun->Eval(xx) > ww) {
         hfb->Fill(xx);
         // Write to file too
         fprintf(fout, "%f\n", xx);
      }
   }
   fclose(fout);

   // Get estimated results per bin and write to a file
   Double_t xx, ex, yy, ey;
   Int_t i = 0;
   for (i = 1; i <= 20; i++) {
      xx = hfb->GetBinCenter(i);
      ex = hfb->GetBinWidth(i) / 2.;
      yy = hfb->GetBinContent(i) / 100000;
      ey = TMath::Sqrt(hfb->GetBinContent(i)) / 100000;
      Printf(" %f %f %f %f", xx, yy, ex, ey);
   }

}
