//
// Small MC program to generate a distribution for the spectra of the pi energy in
// the polarized tau decay tau->pi nu
//

#include <TF1.h>
#include <TH1F.h>
#include <TLorentzRotation.h>
#include <TLorentzVector.h>
#include <TRandom3.h>

const Double_t mZ = 91.1876;
const Double_t mTau = 1.777699;
const Double_t mPi = 0.13957;

static Double_t eTau;
// Fit function
Double_t fitfunction(Double_t *x, Double_t *par)
{
  Float_t xx =x[0];
  Double_t f = par[0] * (1 - par[1] + 2*par[1]/eTau * xx);
  return f;
}

void pienergy(int ndec = 1000, Double_t pol = 0.15) {

   // Parent particle (tau at mZ/2)
   Double_t pzTau = mZ/2.;
   eTau = TMath::Sqrt(pzTau**2 + mTau**2);
   TLorentzVector tau(0.0, 0.0, pzTau, eTau);
   Printf("gamma tau: %f", tau.Gamma());
   TLorentzRotation rtau;
   rtau.Boost(tau.BoostVector());

   // Pi ref values
   Double_t ePi = mTau/2. *(1.-(mPi/mTau)**2);
   Double_t pPi = TMath::Sqrt(ePi**2 - mPi**2);
   // ePi ranges
   Double_t ePiMax = tau.Gamma()*(ePi + tau.Beta() * pPi);
   Double_t ePiMin = tau.Gamma()*(ePi - tau.Beta() * pPi);
   Printf("ePiMax: %f  ePiMin: %f", ePiMax, ePiMin);

   // Init the random generator
   TRandom3 *rndgen = new TRandom3(0);

   // Create the function describing the pi energy distribution in the tau rest frame
   TF1 *epol = new TF1("fbfun","(1+[1]*x)/[0]",-1.,1.);
   epol->SetParameters(2.,pol);

   // Create histo with 20 bins in [-1.,1.]
   hepol = new TH1F("hepol", "hepol", 20, -1., 1.);
   hepi = new TH1F("hepi", "hepi", 40, ePiMin, ePiMax);

   // Fill and draw the histo for test
   Int_t nn = ndec;
   while (nn--) {
      // Generate the energy
      Double_t xx = epol->GetRandom();
      // Fill the test histogram
      hepol->Fill(xx);
      // Fill the 4-vector
      Double_t pzPi = pPi * xx;
      Double_t phi = rndgen->Rndm() * 2 * TMath::Pi();
      Double_t pxPi= TMath::Sqrt(pPi**2 - pzPi**2) * TMath::Cos(phi);
      Double_t pyPi= TMath::Sqrt(pPi**2 - pzPi**2) * TMath::Sin(phi);
      TLorentzVector pi(pxPi, pyPi, pzPi, ePi);
      // Boost it
      pi.Transform(rtau);
      if (nn < 10)
         Printf("px: %f py: %f pz: %f", pi.Px(), pi.Py(), pi.Pz());
      // Fill the energy hist in the lab frame
      hepi->Fill(pi.E());
   }

   // Draw histo 
   hepi->Draw();

   // Create the function for the fit
   TF1 *ffit = new TF1("ffit",fitfunction, 0., mZ/2., 2);
   // Set the initial parameters
   ffit->SetParameters(ndec/2.,pol);
   // Show the result of the fit into the canvas
   gStyle->SetOptFit(1011);
   // Run the fit in the range specified by the function and superimposing the result
   hepi->Fit(ffit, "R", "SAME");
}