//
// This file contains a macro to run locally a generic selector
//

#include "TList.h"
#include "TChain.h"
#include "TKey.h"
#include "TFile.h"
#include "TClass.h"
#include "TNamed.h"
#include "TParameter.h"
#include "TSelector.h"
#include "TString.h"
#include "TStopwatch.h"
#include "TSystem.h"

TChain *chainFromFile(const char *ftxt);
void processLocal(const char *selector, Long64_t evts, TList *input = 0);
void processLocal(TChain *chain, const char *selector, Long64_t evts = 999999999, Long64_t first = 0);
void processLocal(const char *ftxt, const char *selector, Long64_t evts = 999999999, Long64_t first = 0);

void processLocal(const char *selector, Long64_t evts, TList *input)
{
   // Extract the fine name first
   TString selec = selector;
   TString aclic;
   TString args;
   TString io;
   selec = gSystem->SplitAclicMode(selector, aclic, args, io);

   // Create a list, if not existing
   if (!input) input = new TList;

   // Create first an instance of the selector
   selec += aclic;
   TSelector *fSelector = TSelector::GetSelector(selec);
   if (!fSelector) {
      Printf("processLocal: cannot initialize a valid instance of '%s' (input string: %s)", selec.Data(), selector);
      return;
   }

   // Parse arguments: any pair in the form key=value will be passed via the input list
   // as TNamed or TParameter<type>; pairs should be comma- or space-separated 
   TString tkn, key, val;
   Ssiz_t from = 0;
   while (args.Tokenize(tkn, from, "[, ]")) {
      // It must have a '=' (the first one is taken)
      Int_t ie = tkn.Index("=");
      if (ie == kNPOS) continue;
      // Create the list, if not existing
      if (!input) input = new TList;
      // Extract key and val
      key = tkn(0, ie);
      val = tkn(ie+1);
      // The type of input object depends on the type of 'val'
      if (val.IsDigit()) {
         // Use a Long_t type
         input->Add(new TParameter<Long_t>(key, (Long_t)val.Atoi()));
      } else if (val.IsFloat()) {
         // Use a Double_t type
         input->Add(new TParameter<Double_t>(key, val.Atof()));
      } else {
         // Pass is as a string
         input->Add(new TNamed(key, val));
      }
   }

   // Import the input list, if required
   if (input) fSelector->SetInputList(input);

   TStopwatch alltime;

   // Init the selector
   fSelector->Begin(0);
   fSelector->SlaveBegin(0);

   // Run the selector now
   Long64_t e = 0;
   for (e = 0; e < evts; e++) {
      fSelector->Process(e);
   }

   // Terminate
   fSelector->SlaveTerminate();
   fSelector->Terminate();

   // Execution time
   alltime.Print();
}

void processLocal(TChain *chain, const char *selector, Long64_t evts, Long64_t first)
{
   // Local chain processing

   TStopwatch alltime;
   chain->SetCacheSize(30*1024*1024);
   chain->Process(selector, "", evts, first);
   // Execution time
   alltime.Print();
}

void processLocal(const char *ftxt, const char *selector, Long64_t evts, Long64_t first)
{
   // Local processing using the TProofPlayer
   
   TChain *chain = chainFromFile(ftxt);
   if (chain) processLocal(chain, selector, evts, first);
}

TChain *chainFromFile(const char *ftxt)
{
   // Create a TChain for the files listed in 'ftxt'
   // ftxt = filepath#treename 

   TChain *chain = 0;
   TString ffs(ftxt), fn, nm;
   Ssiz_t from  = 0;
   if (ffs.Tokenize(fn, from, "#"))
      ffs.Tokenize(nm, from, "#");
   Printf("fn: %s, tn: %s", fn.Data(), nm.Data());   
   if (gSystem->AccessPathName(fn, kReadPermission)) {
      Printf("File '%s' not available for reading", fn.Data());
      return chain;
   }
   chain = new TChain(nm.Data());
   
   ifstream f;
   f.open(fn.Data());
   if (f.is_open()) {
      while (f.good()) {
         TString line;
         line.ReadToDelim(f);
         // Skip commented or empty lines
         if (!line.IsWhitespace() && !line.BeginsWith("#")) {
            chain->Add(line);
            // Set tree name is not yet done
            if (!chain->GetName() || strlen(chain->GetName()) <= 0) {
               TFile *ff = TFile::Open(line);
               if (ff && !ff->IsZombie() && ff->GetListOfKeys()) {
                  ff->ls();
                  ff->GetListOfKeys()->Print();
                  TIter nxk(ff->GetListOfKeys());
                  TKey *k = 0;
                  while ((k = (TKey *) nxk())) {
                     TClass *cl = TClass::GetClass(k->GetClassName());
                     if (cl && cl->InheritsFrom("TTree")) {
                        chain->SetName(k->GetName());
                        Printf("name set to : %s", chain->GetName());
                        break;
                     }
                  }
               }
               if (ff) delete ff;
            }
         }
      }
      f.close();
   } else {
      Printf("Unable to open file %s", fn.Data());
   }
   chain->ls();
   return chain;
}
